/* $Id: builtin.c,v 1.2 2003/04/10 08:05:19 mulix Exp $ */ 

#include <stddef.h> 
#include <string.h> 
#include <assert.h> 
#include <stdlib.h>
#include <errno.h> 
#include <stdio.h> 

#include "util/util.h" 

#include "include/mush.h" 
#include "include/builtin.h" 

static const struct builtin mush_builtins[] = 
{
	{.cmd = "cd", .func = mush_cd},
	{.cmd = NULL, .func = NULL}
};

static inline size_t minlen(const char* a, const char* b)
{
	size_t al = strlen(a); 
	size_t bl = strlen(b); 

	return ((al < bl) ? al : bl); 
}
static inline void 
verify_builtin_command(const char* cmd, struct command_args* args)
{
	assert(cmd); 
	assert(args); 
	assert(args->argv[0]); 

	assert(!strncmp(cmd, args->argv[0], minlen(cmd, args->argv[0]))); 
} 


const struct builtin* builtin_handler(const char* cmd)
{
	int i = 0; 
	while (i < (sizeof(mush_builtins) / sizeof(mush_builtins[0]))) { 
		const struct builtin* p = &mush_builtins[i]; 
		if (!(p->cmd))
			return NULL; 

		if (!strncmp(p->cmd, cmd, strlen(cmd)))
			return p; 

		++i; 
	}
	return NULL; 
}

int mush_cd(const char* cmd, struct command_args* args)
{
	int ret = 0; 
	char* home = NULL; 

	verify_builtin_command(cmd, args); 

	if ((args->argc == 1) || ((args->argc == 2) && (args->argv[0][0] == '~'))) { 
		home = getenv("HOME"); 

		if (home) { 
			args->argv[1] = home;
			args->argc = 2; 
		}
	}
	
	if ((ret = chdir(args->argv[1]))) { 
		ret = errno; 
		warn("%s: %s", args->argv[1], strerror(errno)); 
	}

	if (home) { 
		args->argv[1] = NULL; 
		free(home); 
	} 
	
	return ret;  
}
